@@ -18,13 +18,11 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-#include <cstdio>
 #include <cstdlib>
 #include <cerrno>
 
-#include "common.h"
-#include "s3fs.h"
 #include "autolock.h"
+#include "s3fs_logger.h"
 
 //-------------------------------------------------------------------
 // Class AutoLock
@@ -64,7 +62,7 @@ AutoLock::~AutoLock()
     if (is_lock_acquired) {
         int result = pthread_mutex_unlock(auto_mutex);
         if(result != 0){
-            S3FS_PRN_CRIT("pthread_mutex_lock returned: %d", result);
+            S3FS_PRN_CRIT("pthread_mutex_unlock returned: %d", result);
             abort();
         }
     }
