@@ -18,16 +18,14 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-#include <cstdio>
-#include <cstdlib>
-#ifndef HAVE_CLOCK_GETTIME
-#include <sys/time.h>
-#endif
-
 #include <algorithm>
+#include <cerrno>
+#include <cstdlib>
+#include <vector>
 
-#include "common.h"
 #include "s3fs.h"
+#include "s3fs_logger.h"
+#include "s3fs_util.h"
 #include "cache.h"
 #include "autolock.h"
 #include "string_util.h"
@@ -35,39 +33,11 @@
 //-------------------------------------------------------------------
 // Utility
 //-------------------------------------------------------------------
-#ifndef CLOCK_REALTIME
-#define CLOCK_REALTIME          0
-#endif
-#ifndef CLOCK_MONOTONIC
-#define CLOCK_MONOTONIC         CLOCK_REALTIME
-#endif
-#ifndef CLOCK_MONOTONIC_COARSE
-#define CLOCK_MONOTONIC_COARSE  CLOCK_MONOTONIC
-#endif
-
-#ifdef HAVE_CLOCK_GETTIME
-static int s3fs_clock_gettime(int clk_id, struct timespec* ts)
-{
-    return clock_gettime(static_cast<clockid_t>(clk_id), ts);
-}
-#else
-static int s3fs_clock_gettime(int clk_id, struct timespec* ts)
-{
-    struct timeval now;
-    if(0 != gettimeofday(&now, NULL)){
-        return -1;
-    }
-    ts->tv_sec  = now.tv_sec;
-    ts->tv_nsec = now.tv_usec * 1000;
-    return 0;
-}
-#endif
-
 inline void SetStatCacheTime(struct timespec& ts)
 {
-    if(-1 == s3fs_clock_gettime(CLOCK_MONOTONIC_COARSE, &ts)){
-        ts.tv_sec  = time(NULL);
-        ts.tv_nsec = 0;
+    if(-1 == clock_gettime(static_cast<clockid_t>(CLOCK_MONOTONIC_COARSE), &ts)){
+        S3FS_PRN_CRIT("clock_gettime failed: %d", errno);
+        abort();
     }
 }
 
@@ -152,7 +122,7 @@ pthread_mutex_t StatCache::stat_cache_lock;
 //-------------------------------------------------------------------
 // Constructor/Destructor
 //-------------------------------------------------------------------
-StatCache::StatCache() : IsExpireTime(true), IsExpireIntervalType(false), ExpireTime(15 * 60), CacheSize(100000), IsCacheNoObject(false)
+StatCache::StatCache() : IsExpireTime(true), IsExpireIntervalType(false), ExpireTime(15 * 60), CacheSize(100000), IsCacheNoObject(true)
 {
     if(this == StatCache::getStatCacheData()){
         stat_cache.clear();
@@ -264,7 +234,7 @@ bool StatCache::GetStat(const std::string& key, struct stat* pst, headers_t* met
             if(ent->noobjcache){
                 if(!IsCacheNoObject){
                     // need to delete this cache.
-                    DelStat(strpath, /*lock_already_held=*/ true);
+                    DelStat(strpath, AutoLock::ALREADY_LOCKED);
                 }else{
                     // noobjcache = true means no object.
                 }
@@ -318,7 +288,7 @@ bool StatCache::GetStat(const std::string& key, struct stat* pst, headers_t* met
     }
 
     if(is_delete_cache){
-        DelStat(strpath, /*lock_already_held=*/ true);
+        DelStat(strpath, AutoLock::ALREADY_LOCKED);
     }
     return false;
 }
@@ -359,12 +329,12 @@ bool StatCache::IsNoObjectCache(const std::string& key, bool overcheck)
     }
 
     if(is_delete_cache){
-        DelStat(strpath, /*lock_already_held=*/ true);
+        DelStat(strpath, AutoLock::ALREADY_LOCKED);
     }
     return false;
 }
 
-bool StatCache::AddStat(const std::string& key, headers_t& meta, bool forcedir, bool no_truncate)
+bool StatCache::AddStat(const std::string& key, const headers_t& meta, bool forcedir, bool no_truncate)
 {
     if(!no_truncate && CacheSize< 1){
         return true;
@@ -402,7 +372,7 @@ bool StatCache::AddStat(const std::string& key, headers_t& meta, bool forcedir,
     ent->meta.clear();
     SetStatCacheTime(ent->cache_date);    // Set time.
     //copy only some keys
-    for(headers_t::iterator iter = meta.begin(); iter != meta.end(); ++iter){
+    for(headers_t::const_iterator iter = meta.begin(); iter != meta.end(); ++iter){
         std::string tag   = lower(iter->first);
         std::string value = iter->second;
         if(tag == "content-type"){
@@ -431,7 +401,7 @@ bool StatCache::AddStat(const std::string& key, headers_t& meta, bool forcedir,
     if(!S_ISLNK(ent->stbuf.st_mode)){
         if(symlink_cache.end() != symlink_cache.find(key)){
             // if symbolic link cache has key, thus remove it.
-            DelSymlink(key.c_str(), true);
+            DelSymlink(key.c_str(), AutoLock::ALREADY_LOCKED);
         }
     }
     return true;
@@ -441,6 +411,9 @@ bool StatCache::AddStat(const std::string& key, headers_t& meta, bool forcedir,
 // Updates only meta data if cached data exists.
 // And when these are updated, it also updates the cache time.
 //
+// Since the file mode may change while the file is open, it is
+// updated as well.
+//
 bool StatCache::UpdateMetaStats(const std::string& key, headers_t& meta)
 {
     if(CacheSize < 1){
@@ -475,6 +448,9 @@ bool StatCache::UpdateMetaStats(const std::string& key, headers_t& meta)
     // Update time.
     SetStatCacheTime(ent->cache_date);
 
+    // Update only mode
+    ent->stbuf.st_mode = get_mode(meta, key);
+
     return true;
 }
 
@@ -528,7 +504,7 @@ bool StatCache::AddNoObjectCache(const std::string& key)
     // check symbolic link cache
     if(symlink_cache.end() != symlink_cache.find(key)){
         // if symbolic link cache has key, thus remove it.
-        DelSymlink(key.c_str(), true);
+        DelSymlink(key.c_str(), AutoLock::ALREADY_LOCKED);
     }
     return true;
 }
@@ -594,7 +570,7 @@ bool StatCache::TruncateCache()
             erase_iters.push_back(iter);
         }
         if(erase_count < erase_iters.size()){
-            sort(erase_iters.begin(), erase_iters.end(), sort_statiterlist());
+            std::sort(erase_iters.begin(), erase_iters.end(), sort_statiterlist());
             while(erase_count < erase_iters.size()){
                 erase_iters.pop_back();
             }
@@ -612,14 +588,14 @@ bool StatCache::TruncateCache()
     return true;
 }
 
-bool StatCache::DelStat(const char* key, bool lock_already_held)
+bool StatCache::DelStat(const char* key, AutoLock::Type locktype)
 {
     if(!key){
         return false;
     }
     S3FS_PRN_INFO3("delete stat cache entry[path=%s]", key);
 
-    AutoLock lock(&StatCache::stat_cache_lock, lock_already_held ? AutoLock::ALREADY_LOCKED : AutoLock::NONE);
+    AutoLock lock(&StatCache::stat_cache_lock, locktype);
 
     stat_cache_t::iterator iter;
     if(stat_cache.end() != (iter = stat_cache.find(std::string(key)))){
@@ -674,7 +650,7 @@ bool StatCache::GetSymlink(const std::string& key, std::string& value)
     }
 
     if(is_delete_cache){
-        DelSymlink(strpath.c_str(), /*lock_already_held=*/ true);
+        DelSymlink(strpath.c_str(), AutoLock::ALREADY_LOCKED);
     }
     return false;
 }
@@ -770,14 +746,14 @@ bool StatCache::TruncateSymlink()
     return true;
 }
 
-bool StatCache::DelSymlink(const char* key, bool lock_already_held)
+bool StatCache::DelSymlink(const char* key, AutoLock::Type locktype)
 {
     if(!key){
         return false;
     }
     S3FS_PRN_INFO3("delete symbolic link cache entry[path=%s]", key);
 
-    AutoLock lock(&StatCache::stat_cache_lock, lock_already_held ? AutoLock::ALREADY_LOCKED : AutoLock::NONE);
+    AutoLock lock(&StatCache::stat_cache_lock, locktype);
 
     symlink_cache_t::iterator iter;
     if(symlink_cache.end() != (iter = symlink_cache.find(std::string(key)))){
@@ -815,12 +791,11 @@ bool convert_header_to_stat(const char* path, const headers_t& meta, struct stat
     if(pst->st_mtime < 0){
         pst->st_mtime = 0L;
     }else{
-#if defined(__APPLE__)
-        pst->st_mtime = mtime.tv_sec;
-#else
-        pst->st_mtim.tv_sec = mtime.tv_sec;
-        pst->st_mtim.tv_nsec = mtime.tv_nsec;
-#endif
+        if(mtime.tv_sec < 0){
+            mtime.tv_sec  = 0;
+            mtime.tv_nsec = 0;
+        }
+        set_timespec_to_stat(*pst, ST_TYPE_MTIME, mtime);
     }
 
     // ctime
@@ -828,12 +803,11 @@ bool convert_header_to_stat(const char* path, const headers_t& meta, struct stat
     if(pst->st_ctime < 0){
         pst->st_ctime = 0L;
     }else{
-#if defined(__APPLE__)
-        pst->st_ctime = ctime.tv_sec;
-#else
-        pst->st_ctim.tv_sec = ctime.tv_sec;
-        pst->st_ctim.tv_nsec = ctime.tv_nsec;
-#endif
+        if(ctime.tv_sec < 0){
+            ctime.tv_sec  = 0;
+            ctime.tv_nsec = 0;
+        }
+        set_timespec_to_stat(*pst, ST_TYPE_CTIME, ctime);
     }
 
     // atime
@@ -841,12 +815,11 @@ bool convert_header_to_stat(const char* path, const headers_t& meta, struct stat
     if(pst->st_atime < 0){
         pst->st_atime = 0L;
     }else{
-#if defined(__APPLE__)
-        pst->st_atime = atime.tv_sec;
-#else
-        pst->st_atim.tv_sec = atime.tv_sec;
-        pst->st_atim.tv_nsec = atime.tv_nsec;
-#endif
+        if(atime.tv_sec < 0){
+            atime.tv_sec  = 0;
+            atime.tv_nsec = 0;
+        }
+        set_timespec_to_stat(*pst, ST_TYPE_ATIME, atime);
     }
 
     // size
