@@ -21,6 +21,9 @@
 #ifndef S3FS_CACHE_H_
 #define S3FS_CACHE_H_
 
+#include <cstring>
+
+#include "autolock.h"
 #include "metaheader.h"
 
 //-------------------------------------------------------------------
@@ -150,13 +153,17 @@ class StatCache
         {
             return GetStat(key, NULL, NULL, overcheck, etag, NULL);
         }
+        bool HasStat(const std::string& key, struct stat* pst, const char* etag)
+        {
+            return GetStat(key, pst, NULL, true, etag, NULL);
+        }
 
         // Cache For no object
         bool IsNoObjectCache(const std::string& key, bool overcheck = true);
         bool AddNoObjectCache(const std::string& key);
 
         // Add stat cache
-        bool AddStat(const std::string& key, headers_t& meta, bool forcedir = false, bool no_truncate = false);
+        bool AddStat(const std::string& key, const headers_t& meta, bool forcedir = false, bool no_truncate = false);
 
         // Update meta stats
         bool UpdateMetaStats(const std::string& key, headers_t& meta);
@@ -165,16 +172,16 @@ class StatCache
         void ChangeNoTruncateFlag(const std::string& key, bool no_truncate);
 
         // Delete stat cache
-        bool DelStat(const char* key, bool lock_already_held = false);
-        bool DelStat(const std::string& key, bool lock_already_held = false)
+        bool DelStat(const char* key, AutoLock::Type locktype = AutoLock::NONE);
+        bool DelStat(const std::string& key, AutoLock::Type locktype = AutoLock::NONE)
         {
-            return DelStat(key.c_str(), lock_already_held);
+            return DelStat(key.c_str(), locktype);
         }
 
         // Cache for symbolic link
         bool GetSymlink(const std::string& key, std::string& value);
         bool AddSymlink(const std::string& key, const std::string& value);
-        bool DelSymlink(const char* key, bool lock_already_held = false);
+        bool DelSymlink(const char* key, AutoLock::Type locktype = AutoLock::NONE);
 };
 
 //-------------------------------------------------------------------
