@@ -21,16 +21,17 @@
 #include <cstdio>
 #include <cstdlib>
 #include <cerrno>
+#include <climits>
 #include <unistd.h>
-#include <sys/types.h>
 #include <dirent.h>
+#include <sys/stat.h>
+#include <sys/statvfs.h>
 
-#include "common.h"
-#include "s3fs.h"
 #include "fdcache.h"
-#include "fdcache_pseudofd.h"
+#include "fdcache_stat.h"
 #include "s3fs_util.h"
 #include "s3fs_logger.h"
+#include "s3fs_cred.h"
 #include "string_util.h"
 #include "autolock.h"
 
@@ -125,7 +126,7 @@ bool FdManager::DeleteCacheDirectory()
         return false;
     }
 
-    std::string mirror_path = FdManager::cache_dir + "/." + bucket + ".mirror";
+    std::string mirror_path = FdManager::cache_dir + "/." + S3fsCred::GetBucket() + ".mirror";
     if(!delete_files_in_dir(mirror_path.c_str(), true)){
         return false;
     }
@@ -181,10 +182,10 @@ bool FdManager::MakeCachePath(const char* path, std::string& cache_path, bool is
     std::string resolved_path(FdManager::cache_dir);
     if(!is_mirror_path){
         resolved_path += "/";
-        resolved_path += bucket;
+        resolved_path += S3fsCred::GetBucket();
     }else{
         resolved_path += "/.";
-        resolved_path += bucket;
+        resolved_path += S3fsCred::GetBucket();
         resolved_path += ".mirror";
     }
 
@@ -208,7 +209,7 @@ bool FdManager::CheckCacheTopDir()
     if(FdManager::cache_dir.empty()){
         return true;
     }
-    std::string toppath(FdManager::cache_dir + "/" + bucket);
+    std::string toppath(FdManager::cache_dir + "/" + S3fsCred::GetBucket());
 
     return check_exist_dir_permission(toppath.c_str());
 }
@@ -319,7 +320,7 @@ bool FdManager::HaveLseekHole()
         }
         FdManager::checked_lseek   = true;
         FdManager::have_lseek_hole = false;
-        return FdManager::have_lseek_hole;
+        return false;
     }
 
     // check SEEK_DATA/SEEK_HOLE options
@@ -336,7 +337,7 @@ bool FdManager::HaveLseekHole()
             result = false;
         }
     }
-    close(fd);
+    fclose(ptmpfp);
 
     FdManager::checked_lseek   = true;
     FdManager::have_lseek_hole = result;
@@ -401,12 +402,22 @@ bool FdManager::HasOpenEntityFd(const char* path)
 
     FdEntity*   ent;
     int         fd = -1;
-    if(NULL == (ent = FdManager::singleton.GetFdEntity(path, fd, false, true))){
+    if(NULL == (ent = FdManager::singleton.GetFdEntity(path, fd, false, AutoLock::ALREADY_LOCKED))){
         return false;
     }
     return (0 < ent->GetOpenCount());
 }
 
+// [NOTE]
+// Returns the number of open pseudo fd.
+//
+int FdManager::GetOpenFdCount(const char* path)
+{
+    AutoLock auto_lock(&FdManager::fd_manager_lock);
+
+	return FdManager::singleton.GetPseudoFdCount(path);
+}
+
 //------------------------------------------------
 // FdManager methods
 //------------------------------------------------
@@ -468,14 +479,14 @@ FdManager::~FdManager()
     }
 }
 
-FdEntity* FdManager::GetFdEntity(const char* path, int& existfd, bool newfd, bool lock_already_held)
+FdEntity* FdManager::GetFdEntity(const char* path, int& existfd, bool newfd, AutoLock::Type locktype)
 {
     S3FS_PRN_INFO3("[path=%s][pseudo_fd=%d]", SAFESTRPTR(path), existfd);
 
     if(!path || '\0' == path[0]){
         return NULL;
     }
-    AutoLock auto_lock(&FdManager::fd_manager_lock, lock_already_held ? AutoLock::ALREADY_LOCKED : AutoLock::NONE);
+    AutoLock auto_lock(&FdManager::fd_manager_lock, locktype);
 
     fdent_map_t::iterator iter = fent.find(std::string(path));
     if(fent.end() != iter && iter->second){
@@ -521,9 +532,9 @@ FdEntity* FdManager::GetFdEntity(const char* path, int& existfd, bool newfd, boo
     return NULL;
 }
 
-FdEntity* FdManager::Open(int& fd, const char* path, headers_t* pmeta, off_t size, time_t time, int flags, bool force_tmpfile, bool is_create, AutoLock::Type type)
+FdEntity* FdManager::Open(int& fd, const char* path, headers_t* pmeta, off_t size, const struct timespec& ts_mctime, int flags, bool force_tmpfile, bool is_create, bool ignore_modify, AutoLock::Type type)
 {
-    S3FS_PRN_DBG("[path=%s][size=%lld][time=%lld][flags=0x%x]", SAFESTRPTR(path), static_cast<long long>(size), static_cast<long long>(time), flags);
+    S3FS_PRN_DBG("[path=%s][size=%lld][ts_mctime=%s][flags=0x%x][force_tmpfile=%s][create=%s][ignore_modify=%s]", SAFESTRPTR(path), static_cast<long long>(size), str(ts_mctime).c_str(), flags, (force_tmpfile ? "yes" : "no"), (is_create ? "yes" : "no"), (ignore_modify ? "yes" : "no"));
 
     if(!path || '\0' == path[0]){
         return NULL;
@@ -551,7 +562,13 @@ FdEntity* FdManager::Open(int& fd, const char* path, headers_t* pmeta, off_t siz
         // found
         ent = iter->second;
 
-        if(ent->IsModified()){
+        // [NOTE]
+        // If the file is being modified and ignore_modify flag is false,
+        // the file size will not be changed even if there is a request
+        // to reduce the size of the modified file.
+        // If you do, the "test_open_second_fd" test will fail.
+        //
+        if(!ignore_modify && ent->IsModified()){
             // If the file is being modified and it's size is larger than size parameter, it will not be resized.
             off_t cur_size = 0;
             if(ent->GetSize(cur_size) && size <= cur_size){
@@ -560,7 +577,7 @@ FdEntity* FdManager::Open(int& fd, const char* path, headers_t* pmeta, off_t siz
         }
 
         // (re)open
-        if(-1 == (fd = ent->Open(pmeta, size, time, flags, type))){
+        if(0 > (fd = ent->Open(pmeta, size, ts_mctime, flags, type))){
             S3FS_PRN_ERR("failed to (re)open and create new pseudo fd for path(%s).", path);
             return NULL;
         }
@@ -576,7 +593,8 @@ FdEntity* FdManager::Open(int& fd, const char* path, headers_t* pmeta, off_t siz
         ent = new FdEntity(path, cache_path.c_str());
 
         // open
-        if(-1 == (fd = ent->Open(pmeta, size, time, flags, type))){
+        if(0 > (fd = ent->Open(pmeta, size, ts_mctime, flags, type))){
+            S3FS_PRN_ERR("failed to open and create new pseudo fd for path(%s).", path);
             delete ent;
             return NULL;
         }
@@ -628,7 +646,7 @@ FdEntity* FdManager::OpenExistFdEntity(const char* path, int& fd, int flags)
     S3FS_PRN_DBG("[path=%s][flags=0x%x]", SAFESTRPTR(path), flags);
 
     // search entity by path, and create pseudo fd
-    FdEntity* ent = Open(fd, path, NULL, -1, -1, flags, false, false, AutoLock::NONE);
+    FdEntity* ent = Open(fd, path, NULL, -1, S3FS_OMIT_TS, flags, false, false, false, AutoLock::NONE);
     if(!ent){
         // Not found entity
         return NULL;
@@ -636,6 +654,29 @@ FdEntity* FdManager::OpenExistFdEntity(const char* path, int& fd, int flags)
     return ent;
 }
 
+// [NOTE]
+// Returns the number of open pseudo fd.
+// This method is called from GetOpenFdCount method which is already locked.
+//
+int FdManager::GetPseudoFdCount(const char* path)
+{
+    S3FS_PRN_DBG("[path=%s]", SAFESTRPTR(path));
+
+    if(!path || '\0' == path[0]){
+        return 0;
+    }
+
+    // search from all entity.
+    for(fdent_map_t::iterator iter = fent.begin(); iter != fent.end(); ++iter){
+        if(iter->second && 0 == strcmp(iter->second->GetPath(), path)){
+            // found the entity for the path
+            return iter->second->GetOpenCount();
+        }
+    }
+    // not found entity
+    return 0;
+}
+
 void FdManager::Rename(const std::string &from, const std::string &to)
 {
     AutoLock auto_lock(&FdManager::fd_manager_lock);
@@ -749,7 +790,7 @@ void FdManager::CleanupCacheDirInternal(const std::string &path)
 {
     DIR*           dp;
     struct dirent* dent;
-    std::string    abs_path = cache_dir + "/" + bucket + path;
+    std::string    abs_path = cache_dir + "/" + S3fsCred::GetBucket() + path;
 
     if(NULL == (dp = opendir(abs_path.c_str()))){
         S3FS_PRN_ERR("could not open cache dir(%s) - errno(%d)", abs_path.c_str(), errno);
@@ -775,7 +816,7 @@ void FdManager::CleanupCacheDirInternal(const std::string &path)
         }else{
             AutoLock auto_lock(&FdManager::fd_manager_lock, AutoLock::NO_WAIT);
             if (!auto_lock.isLockAcquired()) {
-                S3FS_PRN_ERR("could not get fd_manager_lock when clean up file(%s)", next_path.c_str());
+                S3FS_PRN_INFO("could not get fd_manager_lock when clean up file(%s), then skip it.", next_path.c_str());
                 continue;
             }
             fdent_map_t::iterator iter = fent.find(next_path);
