@@ -19,14 +19,15 @@
  */
 
 #include <cstdio>
-#include <cstdlib>
 #include <cerrno>
 #include <unistd.h>
 #include <sstream>
+#include <sys/stat.h>
 
 #include "common.h"
-#include "s3fs.h"
+#include "s3fs_logger.h"
 #include "fdcache_page.h"
+#include "fdcache_stat.h"
 #include "string_util.h"
 
 //------------------------------------------------
@@ -38,12 +39,14 @@ static const int CHECK_CACHEFILE_PART_SIZE = 1024 * 16;    // Buffer size in Pag
 // fdpage_list_t utility
 //------------------------------------------------
 // Inline function for repeated processing
-inline void raw_add_compress_fdpage_list(fdpage_list_t& pagelist, fdpage& page, bool ignore_load, bool ignore_modify, bool default_load, bool default_modify)
+inline void raw_add_compress_fdpage_list(fdpage_list_t& pagelist, const fdpage& orgpage, bool ignore_load, bool ignore_modify, bool default_load, bool default_modify)
 {
-    if(0 < page.bytes){
+    if(0 < orgpage.bytes){
         // [NOTE]
         // The page variable is subject to change here.
         //
+        fdpage page = orgpage;
+
         if(ignore_load){
             page.loaded   = default_load;
         }
@@ -62,65 +65,74 @@ inline void raw_add_compress_fdpage_list(fdpage_list_t& pagelist, fdpage& page,
 // default_modify:  modified flag value in the list after compression when default_modify=true
 //
 // NOTE: ignore_modify and ignore_load cannot both be true.
+//       Zero size pages will be deleted. However, if the page information is the only one,
+//       it will be left behind. This is what you need to do to create a new empty file.
 //
-static fdpage_list_t raw_compress_fdpage_list(const fdpage_list_t& pages, bool ignore_load, bool ignore_modify, bool default_load, bool default_modify)
+static void raw_compress_fdpage_list(const fdpage_list_t& pages, fdpage_list_t& compressed_pages, bool ignore_load, bool ignore_modify, bool default_load, bool default_modify)
 {
-    fdpage_list_t compressed_pages;
-    fdpage        tmppage;
-    bool          is_first = true;
-    for(fdpage_list_t::const_iterator iter = pages.begin(); iter != pages.end(); ++iter){
-        if(!is_first){
-            if( (!ignore_load   && (tmppage.loaded   != iter->loaded  )) ||
-                (!ignore_modify && (tmppage.modified != iter->modified)) )
-            {
-                // Different from the previous area, add it to list
-                raw_add_compress_fdpage_list(compressed_pages, tmppage, ignore_load, ignore_modify, default_load, default_modify);
+    compressed_pages.clear();
 
-                // keep current area
-                tmppage = fdpage(iter->offset, iter->bytes, (ignore_load ? default_load : iter->loaded), (ignore_modify ? default_modify : iter->modified));
-            }else{
-                // Same as the previous area
-                if(tmppage.next() != iter->offset){
-                    // These are not contiguous areas, add it to list
+    fdpage*                 lastpage = NULL;
+    fdpage_list_t::iterator add_iter;
+    for(fdpage_list_t::const_iterator iter = pages.begin(); iter != pages.end(); ++iter){
+        if(0 == iter->bytes){
+            continue;
+        }
+        if(!lastpage){
+            // First item
+            raw_add_compress_fdpage_list(compressed_pages, (*iter), ignore_load, ignore_modify, default_load, default_modify);
+            lastpage = &(compressed_pages.back());
+        }else{
+            // check page continuity
+            if(lastpage->next() != iter->offset){
+                // Non-consecutive with last page, so add a page filled with default values
+                if( (!ignore_load   && (lastpage->loaded   != false)) ||
+                    (!ignore_modify && (lastpage->modified != false)) )
+                {
+                    // add new page
+                    fdpage tmppage(lastpage->next(), (iter->offset - lastpage->next()), false, false);
                     raw_add_compress_fdpage_list(compressed_pages, tmppage, ignore_load, ignore_modify, default_load, default_modify);
 
-                    // keep current area
-                    tmppage = fdpage(iter->offset, iter->bytes, (ignore_load ? default_load : iter->loaded), (ignore_modify ? default_modify : iter->modified));
+                    add_iter = compressed_pages.end();
+                    --add_iter;
+                    lastpage = &(*add_iter);
                 }else{
-                    // These are contiguous areas
-
-                    // add current area
-                    tmppage.bytes += iter->bytes;
+                    // Expand last area
+                    lastpage->bytes = iter->offset - lastpage->offset;
                 }
             }
-        }else{
-            // first erea
-            is_first = false;
 
-            // keep current area
-            tmppage = fdpage(iter->offset, iter->bytes, (ignore_load ? default_load : iter->loaded), (ignore_modify ? default_modify : iter->modified));
+            // add current page
+            if( (!ignore_load   && (lastpage->loaded   != iter->loaded  )) ||
+                (!ignore_modify && (lastpage->modified != iter->modified)) )
+            {
+                // Add new page
+                raw_add_compress_fdpage_list(compressed_pages, (*iter), ignore_load, ignore_modify, default_load, default_modify);
+
+                add_iter = compressed_pages.end();
+                --add_iter;
+                lastpage = &(*add_iter);
+            }else{
+                // Expand last area
+                lastpage->bytes += iter->bytes;
+            }
         }
     }
-    // add last area
-    if(!is_first){
-        raw_add_compress_fdpage_list(compressed_pages, tmppage, ignore_load, ignore_modify, default_load, default_modify);
-    }
-    return compressed_pages;
 }
 
-static fdpage_list_t compress_fdpage_list_ignore_modify(const fdpage_list_t& pages, bool default_modify)
+static void compress_fdpage_list_ignore_modify(const fdpage_list_t& pages, fdpage_list_t& compressed_pages, bool default_modify)
 {
-    return raw_compress_fdpage_list(pages, /* ignore_load= */ false, /* ignore_modify= */ true, /* default_load= */false, /* default_modify= */default_modify);
+    raw_compress_fdpage_list(pages, compressed_pages, /* ignore_load= */ false, /* ignore_modify= */ true, /* default_load= */false, /* default_modify= */default_modify);
 }
 
-static fdpage_list_t compress_fdpage_list_ignore_load(const fdpage_list_t& pages, bool default_load)
+static void compress_fdpage_list_ignore_load(const fdpage_list_t& pages, fdpage_list_t& compressed_pages, bool default_load)
 {
-    return raw_compress_fdpage_list(pages, /* ignore_load= */ true, /* ignore_modify= */ false, /* default_load= */default_load, /* default_modify= */false);
+    raw_compress_fdpage_list(pages, compressed_pages, /* ignore_load= */ true, /* ignore_modify= */ false, /* default_load= */default_load, /* default_modify= */false);
 }
 
-static fdpage_list_t compress_fdpage_list(const fdpage_list_t& pages)
+static void compress_fdpage_list(const fdpage_list_t& pages, fdpage_list_t& compressed_pages)
 {
-    return raw_compress_fdpage_list(pages, /* ignore_load= */ false, /* ignore_modify= */ false, /* default_load= */false, /* default_modify= */false);
+    raw_compress_fdpage_list(pages, compressed_pages, /* ignore_load= */ false, /* ignore_modify= */ false, /* default_load= */false, /* default_modify= */false);
 }
 
 static fdpage_list_t parse_partsize_fdpage_list(const fdpage_list_t& pages, off_t max_partsize)
@@ -293,7 +305,7 @@ bool PageList::CheckAreaInSparseFile(const struct fdpage& checkpage, const fdpag
             check_start = checkpage.offset;
             check_bytes = iter->bytes - (checkpage.offset - iter->offset);
 
-        }else if(iter->offset < (checkpage.offset + checkpage.bytes) && (checkpage.offset + checkpage.bytes) < (iter->offset + iter->bytes)){
+        }else if((checkpage.offset + checkpage.bytes) < (iter->offset + iter->bytes)){  // here, already "iter->offset < (checkpage.offset + checkpage.bytes)" is true.
             // case 3
             check_start = iter->offset;
             check_bytes = checkpage.bytes - (iter->offset - checkpage.offset);
@@ -342,7 +354,7 @@ void PageList::FreeList(fdpage_list_t& list)
     list.clear();
 }
 
-PageList::PageList(off_t size, bool is_loaded, bool is_modified)
+PageList::PageList(off_t size, bool is_loaded, bool is_modified, bool shrinked) : is_shrink(shrinked)
 {
     Init(size, is_loaded, is_modified);
 }
@@ -352,6 +364,7 @@ PageList::PageList(const PageList& other)
     for(fdpage_list_t::const_iterator iter = other.pages.begin(); iter != other.pages.end(); ++iter){
         pages.push_back(*iter);
     }
+    is_shrink = other.is_shrink;
 }
 
 PageList::~PageList()
@@ -362,12 +375,13 @@ PageList::~PageList()
 void PageList::Clear()
 {
     PageList::FreeList(pages);
+    is_shrink = false;
 }
 
 bool PageList::Init(off_t size, bool is_loaded, bool is_modified)
 {
     Clear();
-    if(0 < size){
+    if(0 <= size){
         fdpage page(0, size, is_loaded, is_modified);
         pages.push_back(page);
     }
@@ -385,7 +399,38 @@ off_t PageList::Size() const
 
 bool PageList::Compress()
 {
-    pages = compress_fdpage_list(pages);
+    fdpage* lastpage = NULL;
+    for(fdpage_list_t::iterator iter = pages.begin(); iter != pages.end(); ){
+        if(!lastpage){
+            // First item
+            lastpage = &(*iter);
+            ++iter;
+        }else{
+            // check page continuity
+            if(lastpage->next() != iter->offset){
+                // Non-consecutive with last page, so add a page filled with default values
+                if(lastpage->loaded || lastpage->modified){
+                    // insert new page before current pos
+                    fdpage tmppage(lastpage->next(), (iter->offset - lastpage->next()), false, false);
+                    iter = pages.insert(iter, tmppage);
+                    lastpage = &(*iter);
+                    ++iter;
+                }else{
+                    // Expand last area
+                    lastpage->bytes = iter->offset - lastpage->offset;
+                }
+            }
+            // check current page
+            if(lastpage->loaded == iter->loaded && lastpage->modified == iter->modified){
+                // Expand last area and remove current pos
+                lastpage->bytes += iter->bytes;
+                iter = pages.erase(iter);
+            }else{
+                lastpage = &(*iter);
+                ++iter;
+            }
+        }
+    }
     return true;
 }
 
@@ -411,7 +456,13 @@ bool PageList::Resize(off_t size, bool is_loaded, bool is_modified)
     off_t total = Size();
 
     if(0 == total){
+        // [NOTE]
+        // The is_shrink flag remains unchanged in this function.
+        //
+        bool backup_is_shrink = is_shrink;
+
         Init(size, is_loaded, is_modified);
+        is_shrink = backup_is_shrink;
 
     }else if(total < size){
         // add new area
@@ -431,6 +482,9 @@ bool PageList::Resize(off_t size, bool is_loaded, bool is_modified)
                 }
             }
         }
+        if(is_modified){
+            is_shrink = true;
+        }
     }else{    // total == size
         // nothing to do
     }
@@ -608,9 +662,11 @@ bool PageList::GetPageListsForMultipartUpload(fdpage_list_t& dlpages, fdpage_lis
     }
 
     // make a list by modified flag
-    fdpage_list_t modified_pages = compress_fdpage_list_ignore_load(pages, false);
+    fdpage_list_t modified_pages;
     fdpage_list_t download_pages;         // A non-contiguous page list showing the areas that need to be downloaded
     fdpage_list_t mixupload_pages;        // A continuous page list showing only modified flags for mixupload
+    compress_fdpage_list_ignore_load(pages, modified_pages, false);
+
     fdpage        prev_page;
     for(fdpage_list_t::const_iterator iter = modified_pages.begin(); iter != modified_pages.end(); ++iter){
         if(iter->modified){
@@ -691,8 +747,8 @@ bool PageList::GetPageListsForMultipartUpload(fdpage_list_t& dlpages, fdpage_lis
     }
 
     // compress
-    dlpages    = compress_fdpage_list_ignore_modify(download_pages, false);
-    mixuppages = compress_fdpage_list_ignore_load(mixupload_pages, false);
+    compress_fdpage_list_ignore_modify(download_pages, dlpages, false);
+    compress_fdpage_list_ignore_load(mixupload_pages, mixuppages, false);
 
     // parse by max pagesize
     dlpages    = parse_partsize_fdpage_list(dlpages, max_partsize);
@@ -724,7 +780,7 @@ bool PageList::GetNoDataPageLists(fdpage_list_t& nodata_pages, off_t start, size
 
         fdpage  tmppage;
         tmppage.offset   = std::max(iter->offset, start);
-        tmppage.bytes    = (-1 != stop_pos ? iter->bytes : std::min(iter->bytes, (stop_pos - tmppage.offset)));
+        tmppage.bytes    = (-1 == stop_pos ? iter->bytes : std::min(iter->bytes, (stop_pos - tmppage.offset)));
         tmppage.loaded   = iter->loaded;
         tmppage.modified = iter->modified;
 
@@ -735,7 +791,7 @@ bool PageList::GetNoDataPageLists(fdpage_list_t& nodata_pages, off_t start, size
         nodata_pages.clear();
     }else{
         // compress
-        nodata_pages = compress_fdpage_list(tmp_pagelist);
+        compress_fdpage_list(tmp_pagelist, nodata_pages);
     }
     return true;
 }
@@ -753,6 +809,9 @@ off_t PageList::BytesModified() const
 
 bool PageList::IsModified() const
 {
+    if(is_shrink){
+        return true;
+    }
     for(fdpage_list_t::const_iterator iter = pages.begin(); iter != pages.end(); ++iter){
         if(iter->modified){
             return true;
@@ -763,6 +822,8 @@ bool PageList::IsModified() const
 
 bool PageList::ClearAllModified()
 {
+    is_shrink = false;
+
     for(fdpage_list_t::iterator iter = pages.begin(); iter != pages.end(); ++iter){
         if(iter->modified){
             iter->modified = false;
@@ -926,7 +987,7 @@ void PageList::Dump() const
 {
     int cnt = 0;
 
-    S3FS_PRN_DBG("pages = {");
+    S3FS_PRN_DBG("pages (shrinked=%s) = {", (is_shrink ? "yes" : "no"));
     for(fdpage_list_t::const_iterator iter = pages.begin(); iter != pages.end(); ++iter, ++cnt){
         S3FS_PRN_DBG("  [%08d] -> {%014lld - %014lld : %s / %s}", cnt, static_cast<long long int>(iter->offset), static_cast<long long int>(iter->bytes), iter->loaded ? "loaded" : "unloaded", iter->modified ? "modified" : "not modified");
     }
