@@ -42,6 +42,14 @@ class Semaphore
             dispatch_release(sem);
         }
         void wait() { dispatch_semaphore_wait(sem, DISPATCH_TIME_FOREVER); }
+        bool try_wait()
+        {
+            if(0 == dispatch_semaphore_wait(sem, DISPATCH_TIME_NOW)){
+                return true;
+            }else{
+                return false;
+            }
+        }
         void post() { dispatch_semaphore_signal(sem); }
         int get_value() const { return value; }
 
@@ -67,6 +75,15 @@ class Semaphore
                 r = sem_wait(&mutex);
             } while (r == -1 && errno == EINTR);
         }
+        bool try_wait()
+        {
+            int result;
+            do{
+                result = sem_trywait(&mutex);
+            }while(result == -1 && errno == EINTR);
+
+            return (0 == result);
+        }
         void post() { sem_post(&mutex); }
         int get_value() const { return value; }
 
