@@ -10,7 +10,7 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
@@ -19,12 +19,10 @@
  */
 
 #include <cstdio>
-#include <cstdlib>
 
 #include <string>
 
 #include "common.h"
-#include "s3fs.h"
 #include "s3fs_help.h"
 #include "s3fs_auth.h"
 
@@ -63,9 +61,9 @@ static const char help_string[] =
     "\n"
     "   default_acl (default=\"private\")\n"
     "      - the default canned acl to apply to all written s3 objects,\n"
-    "        e.g., private, public-read.  see\n"
+    "        e.g., private, public-read. see\n"
     "        https://docs.aws.amazon.com/AmazonS3/latest/dev/acl-overview.html#canned-acl\n"
-    "        for the full list of canned acls\n"
+    "        for the full list of canned ACLs\n"
     "\n"
     "   retries (default=\"5\")\n"
     "      - number of times to retry a failed S3 transaction\n"
@@ -85,9 +83,9 @@ static const char help_string[] =
     "      - delete local file cache when s3fs starts and exits.\n"
     "\n"
     "   storage_class (default=\"standard\")\n"
-    "      - store object with specified storage class.  Possible values:\n"
+    "      - store object with specified storage class. Possible values:\n"
     "        standard, standard_ia, onezone_ia, reduced_redundancy,\n"
-    "        intelligent_tiering, glacier, and deep_archive.\n"
+    "        intelligent_tiering, glacier, glacier_ir, and deep_archive.\n"
     "\n"
     "   use_rrs (default is disable)\n"
     "      - use Amazon's Reduced Redundancy Storage.\n"
@@ -204,15 +202,11 @@ static const char help_string[] =
     "      of the stat cache. This option is exclusive with stat_cache_expire,\n"
     "      and is left for compatibility with older versions.\n"
     "\n"
-    "   enable_noobj_cache (default is disable)\n"
-    "      - enable cache entries for the object which does not exist.\n"
-    "      s3fs always has to check whether file (or sub directory) exists \n"
-    "      under object (path) when s3fs does some command, since s3fs has \n"
-    "      recognized a directory which does not exist and has files or \n"
-    "      sub directories under itself. It increases ListBucket request \n"
-    "      and makes performance bad.\n"
-    "      You can specify this option for performance, s3fs memorizes \n"
-    "      in stat cache that the object (file or directory) does not exist.\n"
+    "   disable_noobj_cache (default is enable)\n"
+    "      - By default s3fs memorizes when an object does not exist up until\n"
+    "        the stat cache timeout.  This caching can cause staleness for\n"
+    "        applications.  If disabled, s3fs will not memorize objects and may\n"
+    "        cause extra HeadObject requests and reduce performance.\n"
     "\n"
     "   no_check_certificate\n"
     "      - server certificate won't be checked against the available \n"
@@ -252,20 +246,33 @@ static const char help_string[] =
     "\n"
     "   max_dirty_data (default=\"5120\")\n"
     "      - flush dirty data to S3 after a certain number of MB written.\n"
-    "      The minimum value is 50 MB.  -1 value means disable.\n"
+    "      The minimum value is 50 MB. -1 value means disable.\n"
     "      Cannot be used with nomixupload.\n"
     "\n"
+    "   bucket_size (default=maximum long unsigned integer value)\n"
+    "      - The size of the bucket with which the corresponding\n"
+    "      elements of the statvfs structure will be filled. The option\n"
+    "      argument is an integer optionally followed by a\n"
+    "      multiplicative suffix (GB, GiB, TB, TiB, PB, PiB,\n"
+    "      EB, EiB) (no spaces in between). If no suffix is supplied,\n"
+    "      bytes are assumed; eg: 20000000, 30GB, 45TiB. Note that\n"
+    "      s3fs does not compute the actual volume size (too\n"
+    "      expensive): by default it will assume the maximum possible\n"
+    "      size; however, since this may confuse other software which\n"
+    "      uses s3fs, the advertised bucket size can be set with this\n"
+    "      option.\n"
+    "\n"
     "   ensure_diskfree (default 0)\n"
-    "      - sets MB to ensure disk free space.  This option means the\n"
+    "      - sets MB to ensure disk free space. This option means the\n"
     "        threshold of free space size on disk which is used for the\n"
-    "        cache file by s3fs.  s3fs makes file for\n"
+    "        cache file by s3fs. s3fs makes file for\n"
     "        downloading, uploading and caching files. If the disk free\n"
-    "        space is smaller than this value, s3fs do not use diskspace\n"
+    "        space is smaller than this value, s3fs do not use disk space\n"
     "        as possible in exchange for the performance.\n"
     "\n"
     "   multipart_threshold (default=\"25\")\n"
     "      - threshold, in MB, to use multipart upload instead of\n"
-    "        single-part.  Must be at least 5 MB.\n"
+    "        single-part. Must be at least 5 MB.\n"
     "\n"
     "   singlepart_copy_limit (default=\"512\")\n"
     "      - maximum size, in MB, of a single-part copy before trying \n"
@@ -308,15 +315,35 @@ static const char help_string[] =
     "      mount point by this option like umask.\n"
     "\n"
     "   umask (default is \"0000\")\n"
-    "      - sets umask for files under the mountpoint.  This can allow\n"
+    "      - sets umask for files under the mountpoint. This can allow\n"
     "      users other than the mounting user to read and write to files\n"
     "      that they did not create.\n"
     "\n"
     "   nomultipart (disable multipart uploads)\n"
     "\n"
+    "   streamupload (default is disable)\n"
+    "      - Enable stream upload.\n"
+    "      If this option is enabled, a sequential upload will be performed\n"
+    "      in parallel with the write from the part that has been written\n"
+    "      during a multipart upload.\n"
+    "      This is expected to give better performance than other upload\n"
+    "      functions.\n"
+    "      Note that this option is still experimental and may change in the\n"
+    "      future.\n"
+    "\n"
+    "   max_thread_count (default is \"5\")\n"
+    "      - Specifies the number of threads waiting for stream uploads.\n"
+    "      Note that this option and Streamm Upload are still experimental\n"
+    "      and subject to change in the future.\n"
+    "      This option will be merged with \"parallel_count\" in the future.\n"
+    "\n"
     "   enable_content_md5 (default is disable)\n"
-    "      Allow S3 server to check data integrity of uploads via the\n"
-    "      Content-MD5 header.  This can add CPU overhead to transfers.\n"
+    "      - Allow S3 server to check data integrity of uploads via the\n"
+    "      Content-MD5 header. This can add CPU overhead to transfers.\n"
+    "\n"
+    "   enable_unsigned_payload (default is disable)\n"
+    "      - Do not calculate Content-SHA256 for PutObject and UploadPart\n"
+    "      payloads. This can reduce CPU overhead to transfers.\n"
     "\n"
     "   ecs (default is disable)\n"
     "      - This option instructs s3fs to query the ECS container credential\n"
@@ -330,7 +357,7 @@ static const char help_string[] =
     "\n"
     "   imdsv1only (default is to use IMDSv2 with fallback to v1)\n"
     "      - AWS instance metadata service, used with IAM role authentication,\n"
-    "      supports the use of an API token.  If you're using an IAM role\n"
+    "      supports the use of an API token. If you're using an IAM role\n"
     "      in an environment that does not support IMDSv2, setting this flag\n"
     "      will skip retrieval and usage of the API token when retrieving\n"
     "      IAM credentials.\n"
@@ -343,6 +370,22 @@ static const char help_string[] =
     "   ibm_iam_endpoint (default is https://iam.cloud.ibm.com)\n"
     "      - sets the URL to use for IBM IAM authentication.\n"
     "\n"
+    "   credlib (default=\"\" which means disabled)\n"
+    "      - Specifies the shared library that handles the credentials\n"
+    "      containing the authentication token.\n"
+    "      If this option is specified, the specified credential and token\n"
+    "      processing provided by the shared library ant will be performed\n"
+    "      instead of the built-in credential processing.\n"
+    "      This option cannot be specified with passwd_file, profile,\n"
+    "      use_session_token, ecs, ibm_iam_auth, ibm_iam_endpoint, imdsv1only\n"
+    "      and iam_role option.\n"
+    "\n"
+    "   credlib_opts (default=\"\" which means disabled)\n"
+    "      - Specifies the options to pass when the shared library specified\n"
+    "      in credlib is loaded and then initialized.\n"
+    "      For the string specified in this option, specify the string defined\n"
+    "      by the shared library.\n"
+    "\n"
     "   use_xattr (default is not handling the extended attribute)\n"
     "      Enable to handle the extended attribute (xattrs).\n"
     "      If you set this option, you can use the extended attribute.\n"
@@ -365,15 +408,15 @@ static const char help_string[] =
     "        invalidated even if this option is not specified.\n"
     "\n"
     "   nocopyapi (for other incomplete compatibility object storage)\n"
-    "        For a distributed object storage which is compatibility S3\n"
-    "        API without PUT (copy api).\n"
+    "        Enable compatibility with S3-like APIs which do not support\n"
+    "        PUT (copy api).\n"
     "        If you set this option, s3fs do not use PUT with \n"
     "        \"x-amz-copy-source\" (copy api). Because traffic is increased\n"
     "        2-3 times by this option, we do not recommend this.\n"
     "\n"
     "   norenameapi (for other incomplete compatibility object storage)\n"
-    "        For a distributed object storage which is compatibility S3\n"
-    "        API without PUT (copy api).\n"
+    "        Enable compatibility with S3-like APIs which do not support\n"
+    "        PUT (copy api).\n"
     "        This option is a subset of nocopyapi option. The nocopyapi\n"
     "        option does not use copy-api for all command (ex. chmod, chown,\n"
     "        touch, mv, etc), but this option does not use copy-api for\n"
@@ -412,23 +455,20 @@ static const char help_string[] =
     "        for a object, then the object will not be able to be allowed to\n"
     "        list/modify.\n"
     "\n"
-    "   notsup_compat_dir (not support compatibility directory types)\n"
-    "        As a default, s3fs supports objects of the directory type as\n"
-    "        much as possible and recognizes them as directories.\n"
-    "        Objects that can be recognized as directory objects are \"dir/\",\n"
-    "        \"dir\", \"dir_$folder$\", and there is a file object that does\n"
-    "        not have a directory object but contains that directory path.\n"
-    "        s3fs needs redundant communication to support all these\n"
-    "        directory types. The object as the directory created by s3fs\n"
-    "        is \"dir/\". By restricting s3fs to recognize only \"dir/\" as\n"
-    "        a directory, communication traffic can be reduced. This option\n"
-    "        is used to give this restriction to s3fs.\n"
-    "        However, if there is a directory object other than \"dir/\" in\n"
-    "        the bucket, specifying this option is not recommended. s3fs may\n"
-    "        not be able to recognize the object correctly if an object\n"
-    "        created by s3fs exists in the bucket.\n"
-    "        Please use this option when the directory in the bucket is\n"
-    "        only \"dir/\" object.\n"
+    "   compat_dir (enable support of alternative directory names)\n"
+    "        s3fs supports two different naming schemas \"dir/\" and\n"
+    "        \"dir\" to map directory names to S3 objects and\n"
+    "        vice versa by default. As a third variant, directories can be\n"
+    "        determined indirectly if there is a file object with a path (e.g.\n"
+    "        \"/dir/file\") but without the parent directory.\n"
+    "        This option enables a fourth variant, \"dir_$folder$\", created by\n"
+    "        older applications.\n"
+    "        \n"
+    "        S3fs uses only the first schema \"dir/\" to create S3 objects for\n"
+    "        directories."
+    "        \n"
+    "        The support for these different naming schemas causes an increased\n"
+    "        communication effort.\n"
     "\n"
     "   use_wtf8 - support arbitrary file system encoding.\n"
     "        S3 requires all object names to be valid UTF-8. But some\n"
@@ -456,6 +496,26 @@ static const char help_string[] =
     "        If this file does not exist on macOS, then \"/etc/apache2/mime.types\"\n"
     "        is checked as well.\n"
     "\n"
+    "   proxy (default=\"\")\n"
+    "        This option specifies a proxy to S3 server.\n"
+    "        Specify the proxy with '[<scheme://]hostname(fqdn)[:<port>]' formatted.\n"
+    "        '<schema>://' can be omitted, and 'http://' is used when omitted.\n"
+    "        Also, ':<port>' can also be omitted. If omitted, port 443 is used for\n"
+    "        HTTPS schema, and port 1080 is used otherwise.\n"
+    "        This option is the same as the curl command's '--proxy(-x)' option and\n"
+    "        libcurl's 'CURLOPT_PROXY' flag.\n"
+    "        This option is equivalent to and takes precedence over the environment\n"
+    "        variables 'http_proxy', 'all_proxy', etc.\n"
+    "\n"
+    "   proxy_cred_file (default=\"\")\n"
+    "        This option specifies the file that describes the username and\n"
+    "        passphrase for authentication of the proxy when the HTTP schema\n"
+    "        proxy is specified by the 'proxy' option.\n"
+    "        Username and passphrase are valid only for HTTP schema. If the HTTP\n"
+    "        proxy does not require authentication, this option is not required.\n"
+    "        Separate the username and passphrase with a ':' character and\n"
+    "        specify each as a URL-encoded string.\n"
+    "\n"
     "   logfile - specify the log output file.\n"
     "        s3fs outputs the log file to syslog. Alternatively, if s3fs is\n"
     "        started with the \"-f\" option specified, the log will be output\n"
@@ -470,7 +530,7 @@ static const char help_string[] =
     "        (error), warn (warning), info (information) to debug level.\n"
     "        default debug level is critical. If s3fs run with \"-d\" option,\n"
     "        the debug level is set information. When s3fs catch the signal\n"
-    "        SIGUSR2, the debug level is bumpup.\n"
+    "        SIGUSR2, the debug level is bump up.\n"
     "\n"
     "   curldbg - put curl debug message\n"
     "        Put the debug message from libcurl when this option is specified.\n"
@@ -495,11 +555,17 @@ static const char help_string[] =
     "        check result to that file. The file path parameter can be omitted.\n"
     "        If omitted, the result will be output to stdout or syslog.\n"
     "\n"
+    "   update_parent_dir_stat (default is disable)\n"
+    "        The parent directory's mtime and ctime are updated when a file or\n"
+    "        directory is created or deleted (when the parent directory's inode is\n"
+    "        updated).\n"
+    "        By default, parent directory statistics are not updated.\n"
+    "\n"
     "FUSE/mount Options:\n"
     "\n"
     "   Most of the generic mount options described in 'man mount' are\n"
     "   supported (ro, rw, suid, nosuid, dev, nodev, exec, noexec, atime,\n"
-    "   noatime, sync async, dirsync).  Filesystems are mounted with\n"
+    "   noatime, sync async, dirsync). Filesystems are mounted with\n"
     "   '-onodev,nosuid' by default, which can only be overridden by a\n"
     "   privileged user.\n"
     "   \n"
