@@ -26,17 +26,17 @@
 #include <pwd.h>
 #include <libgen.h>
 #include <dirent.h>
-#include <sys/types.h>
+#include <sys/stat.h>
 #include <sys/utsname.h>
 
 #include <string>
 #include <sstream>
 
-#include "common.h"
-#include "s3fs.h"
+#include "s3fs_logger.h"
 #include "s3fs_util.h"
 #include "string_util.h"
 #include "s3fs_help.h"
+#include "autolock.h"
 
 //-------------------------------------------------------------------
 // Global variables
@@ -170,9 +170,61 @@ int is_uid_include_group(uid_t uid, gid_t gid)
 //-------------------------------------------------------------------
 // Utility for file and directory
 //-------------------------------------------------------------------
+// [NOTE]
+// basename/dirname returns a static variable pointer as the return value.
+// Normally this shouldn't be a problem, but in macos10 we found a case
+// where dirname didn't receive its return value correctly due to thread
+// conflicts.
+// To avoid this, exclusive control is performed by mutex.
+//
+static pthread_mutex_t* pbasename_lock = NULL;
+
+bool init_basename_lock()
+{
+    if(pbasename_lock){
+        S3FS_PRN_ERR("already initialized mutex for posix dirname/basename function.");
+        return false;
+    }
+    pbasename_lock = new pthread_mutex_t;
+
+    pthread_mutexattr_t attr;
+    pthread_mutexattr_init(&attr);
+
+#if S3FS_PTHREAD_ERRORCHECK
+    pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
+#endif
+    int result;
+    if(0 != (result = pthread_mutex_init(pbasename_lock, &attr))){
+        S3FS_PRN_ERR("failed to init pbasename_lock: %d.", result);
+        delete pbasename_lock;
+        pbasename_lock = NULL;
+        return false;
+    }
+    return true;
+}
+
+bool destroy_basename_lock()
+{
+    if(!pbasename_lock){
+        S3FS_PRN_ERR("the mutex for posix dirname/basename function is not initialized.");
+        return false;
+    }
+    int result;
+    if(0 != (result = pthread_mutex_destroy(pbasename_lock))){
+        S3FS_PRN_ERR("failed to destroy pbasename_lock: %d", result);
+        return false;
+    }
+    delete pbasename_lock;
+    pbasename_lock = NULL;
+
+    return true;
+}
+
 std::string mydirname(const std::string& path)
 {
-    return std::string(dirname((char*)path.c_str()));
+    AutoLock auto_lock(pbasename_lock);
+
+    return mydirname(path.c_str());
 }
 
 // safe variant of dirname
@@ -182,12 +234,18 @@ std::string mydirname(const char* path)
     if(!path || '\0' == path[0]){
         return std::string("");
     }
-    return mydirname(std::string(path));
+
+    char *buf = strdup(path);
+    std::string result = dirname(buf);
+    free(buf);
+    return result;
 }
 
 std::string mybasename(const std::string& path)
 {
-    return std::string(basename((char*)path.c_str()));
+    AutoLock auto_lock(pbasename_lock);
+
+    return mybasename(path.c_str());
 }
 
 // safe variant of basename
@@ -197,7 +255,11 @@ std::string mybasename(const char* path)
     if(!path || '\0' == path[0]){
         return std::string("");
     }
-    return mybasename(std::string(path));
+
+    char *buf = strdup(path);
+    std::string result = basename(buf);
+    free(buf);
+    return result;
 }
 
 // mkdir --parents
@@ -392,6 +454,129 @@ void print_launch_message(int argc, char** argv)
     S3FS_PRN_LAUNCH_INFO("%s", message.c_str());
 }
 
+//-------------------------------------------------------------------
+// Utility for nanosecond time(timespec)
+//-------------------------------------------------------------------
+const struct timespec S3FS_OMIT_TS = {0, UTIME_OMIT};
+
+//
+// result: -1  ts1 <  ts2
+//          0  ts1 == ts2
+//          1  ts1 >  ts2
+//
+int compare_timespec(const struct timespec& ts1, const struct timespec& ts2)
+{
+    if(ts1.tv_sec < ts2.tv_sec){
+        return -1;
+    }else if(ts1.tv_sec > ts2.tv_sec){
+        return 1;
+    }else{
+        if(ts1.tv_nsec < ts2.tv_nsec){
+            return -1;
+        }else if(ts1.tv_nsec > ts2.tv_nsec){
+            return 1;
+        }
+    }
+    return 0;
+}
+
+//
+// result: -1  st <  ts
+//          0  st == ts
+//          1  st >  ts
+//
+int compare_timespec(const struct stat& st, stat_time_type type, const struct timespec& ts)
+{
+    struct timespec st_ts;
+    set_stat_to_timespec(st, type, st_ts);
+
+    return compare_timespec(st_ts, ts);
+}
+
+void set_timespec_to_stat(struct stat& st, stat_time_type type, const struct timespec& ts)
+{
+    if(ST_TYPE_ATIME == type){
+        #if defined(__APPLE__)
+            st.st_atime             = ts.tv_sec;
+            st.st_atimespec.tv_nsec = ts.tv_nsec;
+        #else
+            st.st_atim.tv_sec       = ts.tv_sec;
+            st.st_atim.tv_nsec      = ts.tv_nsec;
+        #endif
+    }else if(ST_TYPE_MTIME == type){
+        #if defined(__APPLE__)
+            st.st_mtime             = ts.tv_sec;
+            st.st_mtimespec.tv_nsec = ts.tv_nsec;
+        #else
+            st.st_mtim.tv_sec       = ts.tv_sec;
+            st.st_mtim.tv_nsec      = ts.tv_nsec;
+        #endif
+    }else if(ST_TYPE_CTIME == type){
+        #if defined(__APPLE__)
+            st.st_ctime             = ts.tv_sec;
+            st.st_ctimespec.tv_nsec = ts.tv_nsec;
+        #else
+            st.st_ctim.tv_sec       = ts.tv_sec;
+            st.st_ctim.tv_nsec      = ts.tv_nsec;
+        #endif
+    }else{
+        S3FS_PRN_ERR("unknown type(%d), so skip to set value.", type);
+    }
+}
+
+struct timespec* set_stat_to_timespec(const struct stat& st, stat_time_type type, struct timespec& ts)
+{
+    if(ST_TYPE_ATIME == type){
+        #if defined(__APPLE__)
+           ts.tv_sec  = st.st_atime;
+           ts.tv_nsec = st.st_atimespec.tv_nsec;
+        #else
+           ts         = st.st_atim;
+        #endif
+    }else if(ST_TYPE_MTIME == type){
+        #if defined(__APPLE__)
+           ts.tv_sec  = st.st_mtime;
+           ts.tv_nsec = st.st_mtimespec.tv_nsec;
+        #else
+           ts         = st.st_mtim;
+        #endif
+    }else if(ST_TYPE_CTIME == type){
+        #if defined(__APPLE__)
+           ts.tv_sec  = st.st_ctime;
+           ts.tv_nsec = st.st_ctimespec.tv_nsec;
+        #else
+           ts         = st.st_ctim;
+        #endif
+    }else{
+        S3FS_PRN_ERR("unknown type(%d), so use 0 as timespec.", type);
+        ts.tv_sec     = 0;
+        ts.tv_nsec    = 0;
+    }
+    return &ts;
+}
+
+std::string str_stat_time(const struct stat& st, stat_time_type type)
+{
+    struct timespec ts;
+    return str(*set_stat_to_timespec(st, type, ts));
+}
+
+struct timespec* s3fs_realtime(struct timespec& ts)
+{
+    if(-1 == clock_gettime(static_cast<clockid_t>(CLOCK_REALTIME), &ts)){
+        S3FS_PRN_WARN("failed to clock_gettime by errno(%d)", errno);
+        ts.tv_sec  = time(NULL);
+        ts.tv_nsec = 0;
+    }
+    return &ts;
+}
+
+std::string s3fs_str_realtime()
+{
+    struct timespec ts;
+    return str(*(s3fs_realtime(ts)));
+}
+
 /*
 * Local variables:
 * tab-width: 4
