@@ -21,6 +21,16 @@
 #ifndef S3FS_S3FS_UTIL_H_
 #define S3FS_S3FS_UTIL_H_
 
+#ifndef CLOCK_REALTIME
+#define CLOCK_REALTIME          0
+#endif
+#ifndef CLOCK_MONOTONIC
+#define CLOCK_MONOTONIC         CLOCK_REALTIME
+#endif
+#ifndef CLOCK_MONOTONIC_COARSE
+#define CLOCK_MONOTONIC_COARSE  CLOCK_MONOTONIC
+#endif
+
 //-------------------------------------------------------------------
 // Functions
 //-------------------------------------------------------------------
@@ -30,6 +40,8 @@ void init_sysconf_vars();
 std::string get_username(uid_t uid);
 int is_uid_include_group(uid_t uid, gid_t gid);
 
+bool init_basename_lock();
+bool destroy_basename_lock();
 std::string mydirname(const char* path);
 std::string mydirname(const std::string& path);
 std::string mybasename(const char* path);
@@ -44,6 +56,24 @@ bool compare_sysname(const char* target);
 
 void print_launch_message(int argc, char** argv);
 
+//
+// Utility for nanosecond time(timespec)
+//
+enum stat_time_type{
+    ST_TYPE_ATIME,
+    ST_TYPE_MTIME,
+    ST_TYPE_CTIME
+};
+extern const struct timespec S3FS_OMIT_TS;
+
+int compare_timespec(const struct timespec& ts1, const struct timespec& ts2);
+int compare_timespec(const struct stat& st, stat_time_type type, const struct timespec& ts);
+void set_timespec_to_stat(struct stat& st, stat_time_type type, const struct timespec& ts);
+struct timespec* set_stat_to_timespec(const struct stat& st, stat_time_type type, struct timespec& ts);
+std::string str_stat_time(const struct stat& st, stat_time_type type);
+struct timespec* s3fs_realtime(struct timespec& ts);
+std::string s3fs_str_realtime();
+
 #endif // S3FS_S3FS_UTIL_H_
 
 /*
