@@ -174,7 +174,56 @@ enum signature_type_t {
 //----------------------------------------------
 // etaglist_t / filepart / untreatedpart
 //----------------------------------------------
-typedef std::list<std::string> etaglist_t;
+//
+// Etag string and part number pair
+//
+struct etagpair
+{
+    std::string  etag;        // expected etag value
+    int          part_num;    // part number
+
+    explicit etagpair(const char* petag = NULL, int part = -1) : etag(petag ? petag : ""), part_num(part) {}
+
+    ~etagpair()
+    {
+      clear();
+    }
+
+    void clear()
+    {
+        etag.erase();
+        part_num = -1;
+    }
+};
+
+typedef std::list<etagpair> etaglist_t;
+
+struct petagpool
+{
+    std::list<etagpair*> petaglist;
+
+    ~petagpool()
+    {
+        clear();
+    }
+
+    void clear()
+    {
+        for(std::list<etagpair*>::iterator it = petaglist.begin(); petaglist.end() != it; ++it){
+            if(*it){
+                delete (*it);
+            }
+        }
+        petaglist.clear();
+    }
+
+    etagpair* add(const etagpair& etag_entity)
+    {
+        etagpair* petag = new etagpair(etag_entity);
+        petaglist.push_back(petag);
+        return petag;
+    }
+};
 
 //
 // Each part information for Multipart upload
@@ -187,9 +236,9 @@ struct filepart
     off_t        startpos;    // seek fd point for uploading
     off_t        size;        // uploading size
     bool         is_copy;     // whether is copy multipart
-    std::string* petag;       // use only parallel upload
+    etagpair*    petag;       // use only parallel upload
 
-    filepart(bool is_uploaded = false, int _fd = -1, off_t part_start = 0, off_t part_size = -1, bool is_copy_part = false, std::string* petag = NULL) : uploaded(false), fd(_fd), startpos(part_start), size(part_size), is_copy(is_copy_part), petag(petag) {}
+    explicit filepart(bool is_uploaded = false, int _fd = -1, off_t part_start = 0, off_t part_size = -1, bool is_copy_part = false, etagpair* petagpair = NULL) : uploaded(false), fd(_fd), startpos(part_start), size(part_size), is_copy(is_copy_part), petag(petagpair) {}
 
     ~filepart()
     {
@@ -207,16 +256,27 @@ struct filepart
         petag    = NULL;
     }
 
-    void add_etag_list(etaglist_t* list)
+    void add_etag_list(etaglist_t& list, int partnum = -1)
     {
-        list->push_back(std::string());
-        petag = &list->back();
+        if(-1 == partnum){
+            partnum = static_cast<int>(list.size()) + 1;
+        }
+        list.push_back(etagpair(NULL, partnum));
+        petag = &list.back();
     }
 
-    void add_etag(std::string* petagobj)
+    void set_etag(etagpair* petagobj)
     {
         petag = petagobj;
     }
+
+    int get_part_number() const
+    {
+        if(!petag){
+            return -1;
+        }
+        return petag->part_num;
+    }
 };
 
 typedef std::list<filepart> filepart_list_t;
@@ -230,7 +290,7 @@ struct untreatedpart
     off_t size;             // number of untreated bytes
     long  untreated_tag;    // untreated part tag
 
-    untreatedpart(off_t part_start = 0, off_t part_size = 0, long part_untreated_tag = 0) : start(part_start), size(part_size), untreated_tag(part_untreated_tag)
+    explicit untreatedpart(off_t part_start = 0, off_t part_size = 0, long part_untreated_tag = 0) : start(part_start), size(part_size), untreated_tag(part_untreated_tag)
     {
         if(part_start < 0 || part_size <= 0){
             clear();        // wrong parameter, so clear value.
@@ -249,9 +309,13 @@ struct untreatedpart
         untreated_tag = 0;
     }
 
+    // [NOTE]
+    // Check if the areas overlap
+    // However, even if the areas do not overlap, this method returns true if areas are adjacent.
+    //
     bool check_overlap(off_t chk_start, off_t chk_size)
     {
-        if(chk_start < 0 || chk_size <= 0 || (chk_start + chk_size) < start || (start + size) < chk_start){
+        if(chk_start < 0 || chk_size <= 0 || start < 0 || size <= 0 || (chk_start + chk_size) < start || (start + size) < chk_start){
             return false;
         }
         return true;
@@ -275,6 +339,29 @@ struct untreatedpart
 
 typedef std::list<untreatedpart> untreated_list_t;
 
+//
+// Information on each part of multipart upload
+//
+struct mp_part
+{
+    off_t  start;
+    off_t  size;
+    int    part_num;        // Set only for information to upload
+
+    explicit mp_part(off_t set_start = 0, off_t set_size = 0, int part = 0) : start(set_start), size(set_size), part_num(part) {}
+};
+
+typedef std::list<struct mp_part> mp_part_list_t;
+
+inline off_t total_mp_part_list(const mp_part_list_t& mplist)
+{
+    off_t size = 0;
+    for(mp_part_list_t::const_iterator iter = mplist.begin(); iter != mplist.end(); ++iter){
+        size += iter->size;
+    }
+    return size;
+}
+
 //-------------------------------------------------------------------
 // mimes_t
 //-------------------------------------------------------------------
